<?php
namespace Dolyame\Payment\Orm;

use Bitrix\Main\Localization\Loc,
	Bitrix\Main\ORM\Data\DataManager,
	Bitrix\Main\ORM\Fields\DatetimeField,
	Bitrix\Main\ORM\Fields\FloatField,
	Bitrix\Main\ORM\Fields\IntegerField,
	Bitrix\Main\ORM\Fields\StringField,
	Bitrix\Main\ORM\Fields\TextField,
	Bitrix\Main\ORM\Fields\Validators\LengthValidator;

Loc::loadMessages(__FILE__);

/**
 * Class PaymentTable
 *
 * Fields:
 * <ul>
 * <li> id string(100) mandatory
 * <li> order_number string(100) mandatory
 * <li> order_id int mandatory
 * <li> payment_id int mandatory
 * <li> status string(20) mandatory
 * <li> items text mandatory
 * <li> receipt text optional
 * <li> date datetime mandatory
 * <li> amount double mandatory
 * <li> refund_id string(36) optional
 * <li> payment_schedule text optional
 * </ul>
 *
 * @package Bitrix\Payment
 **/

class PaymentTable extends DataManager
{
	/**
	 * Returns DB table name for entity.
	 *
	 * @return string
	 */
	public static function getTableName()
	{
		return 'dolyame_payment';
	}

	/**
	 * Returns entity map definition.
	 *
	 * @return array
	 */
	public static function getMap()
	{
		return [
			new StringField(
				'id',
				[
					'primary' => true,
					'validation' => [__CLASS__, 'validateId'],
					'title' => Loc::getMessage('PAYMENT_ENTITY_ID_FIELD')
				]
			),
			new StringField(
				'order_number',
				[
					'required' => true,
					'validation' => [__CLASS__, 'validateOrderNumber'],
					'title' => Loc::getMessage('PAYMENT_ENTITY_ORDER_NUMBER_FIELD')
				]
			),
			new IntegerField(
				'order_id',
				[
					'required' => true,
					'title' => Loc::getMessage('PAYMENT_ENTITY_ORDER_ID_FIELD')
				]
			),
			new IntegerField(
				'payment_id',
				[
					'required' => true,
					'title' => Loc::getMessage('PAYMENT_ENTITY_PAYMENT_ID_FIELD')
				]
			),
			new StringField(
				'status',
				[
					'required' => true,
					'validation' => [__CLASS__, 'validateStatus'],
					'title' => Loc::getMessage('PAYMENT_ENTITY_STATUS_FIELD')
				]
			),
			new TextField(
				'items',
				[
					'required' => true,
					'title' => Loc::getMessage('PAYMENT_ENTITY_ITEMS_FIELD')
				]
			),
			new TextField(
				'receipt',
				[
					'title' => Loc::getMessage('PAYMENT_ENTITY_RECEIPT_FIELD')
				]
			),
			new DatetimeField(
				'date',
				[
					'required' => true,
					'title' => Loc::getMessage('PAYMENT_ENTITY_DATE_FIELD')
				]
			),
			new FloatField(
				'amount',
				[
					'required' => true,
					'title' => Loc::getMessage('PAYMENT_ENTITY_AMOUNT_FIELD')
				]
			),
			new StringField(
				'refund_id',
				[
					'validation' => [__CLASS__, 'validateRefundId'],
					'title' => Loc::getMessage('PAYMENT_ENTITY_REFUND_ID_FIELD')
				]
			),
			new TextField(
				'payment_schedule',
				[
					'title' => Loc::getMessage('PAYMENT_ENTITY_PAYMENT_SCHEDULE_FIELD')
				]
			),
		];
	}

	/**
	 * Returns validators for id field.
	 *
	 * @return array
	 */
	public static function validateId()
	{
		return [
			new LengthValidator(null, 100),
		];
	}

	/**
	 * Returns validators for order_number field.
	 *
	 * @return array
	 */
	public static function validateOrderNumber()
	{
		return [
			new LengthValidator(null, 100),
		];
	}

	/**
	 * Returns validators for status field.
	 *
	 * @return array
	 */
	public static function validateStatus()
	{
		return [
			new LengthValidator(null, 20),
		];
	}

	/**
	 * Returns validators for refund_id field.
	 *
	 * @return array
	 */
	public static function validateRefundId()
	{
		return [
			new LengthValidator(null, 36),
		];
	}
}